
#include "application.h"
#include <gdk/gdk.h>
#include "basewindow.h"
#include "documentwindow.h"

struct _ApplicationPrivate {
	GSList* window_list;
	int _win_count;
};
#define APPLICATION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_APPLICATION, ApplicationPrivate))
enum  {
	APPLICATION_DUMMY_PROPERTY,
	APPLICATION_WIN_COUNT,
};
static char* application_config_dir;
static void application_url_hook (GtkAboutDialog* dialog, char* link, gpointer data);
static void application_email_hook (GtkAboutDialog* dialog, char* email, gpointer data);
static void application_open_project (Application* self, char* startup_id);
static void __lambda0 (GtkObject* w, Application* self);
static void application_open_document (Application* self, char* uri, char* startup_id);
static void application_add_documents (Application* self, int docs_length1, char** docs);
static int application_main (int args_length1, char** args);
static void application_on_message_received (GUniqueApp* app, GUniqueAppCommand command, char* data, char* startup_id, unsigned int workspace, Application* self);
static void application_dispose (GObject * obj);

char* application_get_config_dir ()
{
	return application_config_dir;
}


Application* application_new (char** args)
{
	GParameter * __params = g_new0 (GParameter, 0);
	GParameter * __params_it = __params;
	Application * self = g_object_newv (TYPE_APPLICATION, __params_it - __params, __params);
	int i;
	char* startup_id = g_getenv ("DESKTOP_STARTUP_ID");
	gtk_init (NULL, &args);
	gnome_vfs_init ();
	GUniqueApp* instance = g_unique_app_get_with_startup_id ("scratchpad", startup_id);
	if (g_unique_app_is_running (instance)) {
		gboolean first = TRUE;
		i = 0;
		{
			char** doc_it;
			for (doc_it = args; *doc_it != NULL; doc_it = doc_it + 1) {
				char* doc = *doc_it;
				{
					if (first) {
						first = FALSE;
						continue;
					}
					g_unique_app_open_uri (instance, doc);
					i++;
				}
			}
		}
		if (i == 0) {
			gdk_notify_startup_complete ();
			g_unique_app_new_document (instance);
		}
		gdk_notify_startup_complete ();
	} else {
		g_signal_connect_object (instance, "message", (GCallback) application_on_message_received, self, 0);
		application_add_documents (self, -1, args);
	}
	if (application_get_win_count (self) > 0) {
		gtk_about_dialog_set_url_hook (application_url_hook, NULL, NULL);
		gtk_about_dialog_set_email_hook (application_email_hook, NULL, NULL);
		gtk_main ();
	}
	gnome_vfs_shutdown ();
	return self;
}


static void application_url_hook (GtkAboutDialog* dialog, char* link, gpointer data)
{
	g_return_if_fail (dialog == NULL || GTK_IS_ABOUT_DIALOG (dialog));
	char* __temp1 = NULL;
	g_spawn_command_line_async ((__temp1 = g_strconcat ("gnome-open ", link, NULL)), NULL);
	(__temp1 == NULL ? NULL : (__temp1 = (g_free (__temp1), NULL)));
}


static void application_email_hook (GtkAboutDialog* dialog, char* email, gpointer data)
{
	g_return_if_fail (dialog == NULL || GTK_IS_ABOUT_DIALOG (dialog));
	char* __temp2 = NULL;
	g_spawn_command_line_async ((__temp2 = g_strconcat ("gnome-open mailto:", email, NULL)), NULL);
	(__temp2 == NULL ? NULL : (__temp2 = (g_free (__temp2), NULL)));
}


static void application_open_project (Application* self, char* startup_id)
{
	g_return_if_fail (IS_APPLICATION (self));
	/* open a project window when no arguments have been passed to the process
	 this is not implemented yet...*/
	char* message = "This would create a new project oriented window, but this feature is not yet implemented. Please use Scratchpad from your file manager or terminal to open a document directly instead.";
	GtkMessageDialog* d = g_object_ref_sink (gtk_message_dialog_new (NULL, (GtkDialogFlags) 0, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, message));
	if (startup_id != NULL) {
		gtk_window_set_startup_id (GTK_WINDOW (d), startup_id);
	}
	/* this is blocking which kinda sucks, but so does the entire existance of this dialog*/
	gtk_dialog_run (GTK_DIALOG (d));
	g_signal_emit_by_name (G_OBJECT (d), "destroy");
	(d == NULL ? NULL : (d = (g_object_unref (d), NULL)));
}


static void __lambda0 (GtkObject* w, Application* self)
{
	g_return_if_fail (w == NULL || GTK_IS_OBJECT (w));
	self->priv->window_list = g_slist_remove (self->priv->window_list, w);
	application_set_win_count (self, application_get_win_count (self) - 1);
}


static void application_open_document (Application* self, char* uri, char* startup_id)
{
	g_return_if_fail (IS_APPLICATION (self));
	{
		GSList* window_it;
		for (window_it = self->priv->window_list; window_it != NULL; window_it = window_it->next) {
			BaseWindow* window = window_it->data;
			{
				if (base_window_is_document (window, uri)) {
					gtk_window_present_with_time (GTK_WINDOW (window), time (NULL));
					return;
				}
			}
		}
	}
	application_set_win_count (self, application_get_win_count (self) + 1);
	DocumentWindow* w = g_object_ref_sink (document_window_new (uri));
	if (startup_id != NULL) {
		gtk_window_set_startup_id (GTK_WINDOW (w), startup_id);
	}
	gtk_widget_show_all (GTK_WIDGET (w));
	DocumentWindow* __temp3 = NULL;
	self->priv->window_list = g_slist_append (self->priv->window_list, (__temp3 = w, (__temp3 == NULL ? __temp3 : g_object_ref (__temp3))));
	g_signal_connect_object (w, "destroy", (GCallback) __lambda0, self, 0);
	(w == NULL ? NULL : (w = (g_object_unref (w), NULL)));
}


static void application_add_documents (Application* self, int docs_length1, char** docs)
{
	g_return_if_fail (IS_APPLICATION (self));
	int i = 0;
	gboolean first = TRUE;
	{
		char** uri_it;
		for (uri_it = docs; *uri_it != NULL; uri_it = uri_it + 1) {
			char* uri = *uri_it;
			{
				if (first) {
					first = FALSE;
					continue;
				}
				application_open_document (self, uri, NULL);
				i++;
			}
		}
	}
	if (i == 0) {
		application_open_project (self, NULL);
	}
}


static int application_main (int args_length1, char** args)
{
	char* __temp4 = NULL;
	application_config_dir = (__temp4 = g_strconcat (g_get_home_dir (), "/.gnome2/scratchpad", NULL), (application_config_dir == NULL ? NULL : (application_config_dir = (g_free (application_config_dir), NULL))), __temp4);
	char* __temp5 = NULL;
	g_mkdir_with_parents ((__temp5 = g_strconcat (application_get_config_dir (), "/metadata/", NULL)), 0750);
	(__temp5 == NULL ? NULL : (__temp5 = (g_free (__temp5), NULL)));
	Application* app = application_new (args);
	(app == NULL ? NULL : (app = (g_object_unref (app), NULL)));
}


int main (int argc, char ** argv)
{
	g_type_init ();
	return application_main (argc, argv);
}


static void application_on_message_received (GUniqueApp* app, GUniqueAppCommand command, char* data, char* startup_id, unsigned int workspace, Application* self)
{
	g_return_if_fail (IS_APPLICATION (self));
	g_return_if_fail (app == NULL || G_IS_UNIQUE_APP (app));
	if (command == G_UNIQUE_APP_OPEN) {
		application_open_document (self, data, startup_id);
	} else {
		if (command == G_UNIQUE_APP_NEW) {
			application_open_project (self, startup_id);
		}
	}
}


int application_get_win_count (Application* self)
{
	g_return_val_if_fail (IS_APPLICATION (self), 0);
	return self->priv->_win_count;
}


void application_set_win_count (Application* self, int value)
{
	g_return_if_fail (IS_APPLICATION (self));
	self->priv->_win_count = value;
	if (self->priv->_win_count <= 0) {
		gtk_main_quit ();
	}
}


static void application_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	Application * self = APPLICATION (object);
	switch (property_id) {
		case APPLICATION_WIN_COUNT:
		g_value_set_int (value, application_get_win_count (self));
		break;
	}
}


static void application_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	Application * self = APPLICATION (object);
	switch (property_id) {
		case APPLICATION_WIN_COUNT:
		application_set_win_count (self, g_value_get_int (value));
		break;
	}
}


static void application_class_init (ApplicationClass * klass)
{
	g_type_class_add_private (klass, sizeof (ApplicationPrivate));
	G_OBJECT_CLASS (klass)->get_property = application_get_property;
	G_OBJECT_CLASS (klass)->set_property = application_set_property;
	G_OBJECT_CLASS (klass)->dispose = application_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), APPLICATION_WIN_COUNT, g_param_spec_int ("win-count", "foo", "bar", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void application_init (Application * self)
{
	self->priv = APPLICATION_GET_PRIVATE (self);
	self->priv->_win_count = 0;
}


static void application_dispose (GObject * obj)
{
	Application * self = APPLICATION (obj);
	(self->priv->window_list == NULL ? NULL : (self->priv->window_list = (g_slist_free (self->priv->window_list), NULL)));
	ApplicationClass * klass;
	GObjectClass * parent_class;
	klass = APPLICATION_CLASS (g_type_class_peek (TYPE_APPLICATION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType application_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ApplicationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) application_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Application), 0, (GInstanceInitFunc) application_init };
		g_define_type_id = g_type_register_static (G_TYPE_OBJECT, "Application", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



