
#include "document.h"
#include <gtksourceview/gtksourcelanguagesmanager.h>
#include <gtksourceview/gtksourcelanguage.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include "application.h"

struct _DocumentPrivate {
	char* file_name;
	char* mime_type;
	char* modified;
	MetadataManager* _metadata;
};
#define DOCUMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_DOCUMENT, DocumentPrivate))
enum  {
	DOCUMENT_DUMMY_PROPERTY,
	DOCUMENT_METADATA,
};
static GObject * document_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void document_dispose (GObject * obj);

Document* document_new (char* uri)
{
	GParameter * __params = g_new0 (GParameter, 0);
	GParameter * __params_it = __params;
	Document * self = g_object_newv (TYPE_DOCUMENT, __params_it - __params, __params);
	char* __temp0 = NULL;
	self->priv->file_name = (__temp0 = document_uri_to_filename (uri), (self->priv->file_name == NULL ? NULL : (self->priv->file_name = (g_free (self->priv->file_name), NULL))), __temp0);
	MetadataManager* __temp1 = NULL;
	document_set_metadata (self, (__temp1 = metadata_manager_new (gnome_vfs_escape_string (self->priv->file_name))));
	(__temp1 == NULL ? NULL : (__temp1 = (g_object_unref (__temp1), NULL)));
	char* __temp3 = NULL;
	char* __temp2 = NULL;
	self->priv->mime_type = (__temp3 = (__temp2 = gnome_vfs_get_mime_type_for_name (self->priv->file_name), (__temp2 == NULL ? __temp2 : g_strdup (__temp2))), (self->priv->mime_type == NULL ? NULL : (self->priv->mime_type = (g_free (self->priv->mime_type), NULL))), __temp3);
	if (self->priv->mime_type != NULL) {
		GtkSourceLanguagesManager* slm = gtk_source_languages_manager_new ();
		GtkSourceLanguage* lang = gtk_source_languages_manager_get_language_from_mime_type (slm, self->priv->mime_type);
		if (lang != NULL) {
			gtk_source_buffer_set_language (GTK_SOURCE_BUFFER (self), lang);
		}
		(slm == NULL ? NULL : (slm = (g_object_unref (slm), NULL)));
	}
	return self;
}


gboolean document_open (Document* self)
{
	g_return_val_if_fail (IS_DOCUMENT (self), FALSE);
	/* TODO: error dialogs*/
	char* contents = NULL;
	if (!g_file_get_contents (self->priv->file_name, &contents, NULL, NULL)) {
		gboolean __temp4;
		return (__temp4 = FALSE, (contents == NULL ? NULL : (contents = (g_free (contents), NULL))), __temp4);
	}
	if (!g_utf8_validate (contents, -1, NULL)) {
		/* this is lame, but everyone should use UTF-8 anyway...*/
		char* __temp5 = NULL;
		contents = (__temp5 = g_convert (contents, -1, "UTF-8", "ISO-8859-2", NULL, NULL, NULL), (contents == NULL ? NULL : (contents = (g_free (contents), NULL))), __temp5);
		if (!g_utf8_validate (contents, -1, NULL)) {
			gboolean __temp6;
			return (__temp6 = FALSE, (contents == NULL ? NULL : (contents = (g_free (contents), NULL))), __temp6);
		}
	}
	gtk_source_buffer_begin_not_undoable_action (GTK_SOURCE_BUFFER (self));
	gtk_text_buffer_set_text (GTK_TEXT_BUFFER (self), contents, -1);
	gtk_source_buffer_end_not_undoable_action (GTK_SOURCE_BUFFER (self));
	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (self), FALSE);
	gboolean __temp7;
	return (__temp7 = TRUE, (contents == NULL ? NULL : (contents = (g_free (contents), NULL))), __temp7);
	(contents == NULL ? NULL : (contents = (g_free (contents), NULL)));
}


void document_save (Document* self)
{
	g_return_if_fail (IS_DOCUMENT (self));
	g_return_if_fail (self != NULL);
	if (!gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (self))) {
		return;
	}
	char* buffer = NULL;
	char* __temp8 = NULL;
	buffer = (__temp8 = g_strconcat (application_get_config_dir (), "/save_buffer", NULL), (buffer == NULL ? NULL : (buffer = (g_free (buffer), NULL))), __temp8);
	GtkTextIter start;
	GtkTextIter end;
	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (self), &start, &end);
	char* contents = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (self), &start, &end, FALSE);
	GError* err = NULL;
	if (!g_file_set_contents (buffer, contents, -1, &err)) {
		char* __temp9 = NULL;
		fprintf (stdout, (__temp9 = g_strconcat ("Error: Could not save document: ", err->message, NULL)), NULL);
		(__temp9 == NULL ? NULL : (__temp9 = (g_free (__temp9), NULL)));
		(buffer == NULL ? NULL : (buffer = (g_free (buffer), NULL)));
		return;
	}
	if (g_rename (buffer, self->priv->file_name) == -1) {
		char* __temp10 = NULL;
		fprintf (stdout, (__temp10 = g_strconcat ("Error: Could not save document: ", err->message, NULL)), NULL);
		(__temp10 == NULL ? NULL : (__temp10 = (g_free (__temp10), NULL)));
		(buffer == NULL ? NULL : (buffer = (g_free (buffer), NULL)));
		return;
	}
	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (self), FALSE);
	(buffer == NULL ? NULL : (buffer = (g_free (buffer), NULL)));
}


char* document_get_file_name (Document* self)
{
	g_return_val_if_fail (IS_DOCUMENT (self), NULL);
	return self->priv->file_name;
}


char* document_get_mimetype (Document* self)
{
	g_return_val_if_fail (IS_DOCUMENT (self), NULL);
	return self->priv->mime_type;
}


char* document_get_folder (Document* self)
{
	g_return_val_if_fail (IS_DOCUMENT (self), NULL);
	char** parts = g_strsplit (g_utf8_strreverse (self->priv->file_name, -1), "/", 2);
	char* __temp13 = NULL;
	return (__temp13 = g_utf8_strreverse (parts[1], -1), (parts == NULL ? NULL : (parts = (g_strfreev (parts), NULL))), __temp13);
	(parts == NULL ? NULL : (parts = (g_strfreev (parts), NULL)));
}


char* document_get_basename (Document* self)
{
	g_return_val_if_fail (IS_DOCUMENT (self), NULL);
	return g_filename_display_basename (self->priv->file_name);
}


char* document_get_title_string (Document* self)
{
	g_return_val_if_fail (IS_DOCUMENT (self), NULL);
	char* __temp15 = NULL;
	char* folder = (__temp15 = document_get_folder (self), (__temp15 == NULL ? __temp15 : g_strdup (__temp15)));
	char* home = g_get_home_dir ();
	if (g_utf8_collate (folder, home) == 0) {
		char* __temp16 = NULL;
		folder = (__temp16 = g_strdup (""), (folder == NULL ? NULL : (folder = (g_free (folder), NULL))), __temp16);
	} else {
		if (g_str_has_prefix (folder, home)) {
			char* __temp18 = NULL;
			char* __temp17 = NULL;
			folder = (__temp18 = g_strconcat ((__temp17 = g_strconcat (" (", g_utf8_offset_to_pointer (folder, g_utf8_strlen (home, -1) + 1), NULL)), ")", NULL), (folder == NULL ? NULL : (folder = (g_free (folder), NULL))), __temp18);
			(__temp17 == NULL ? NULL : (__temp17 = (g_free (__temp17), NULL)));
		} else {
			char* __temp20 = NULL;
			char* __temp19 = NULL;
			folder = (__temp20 = g_strconcat ((__temp19 = g_strconcat (" (", folder, NULL)), ")", NULL), (folder == NULL ? NULL : (folder = (g_free (folder), NULL))), __temp20);
			(__temp19 == NULL ? NULL : (__temp19 = (g_free (__temp19), NULL)));
		}
	}
	char* __temp21 = NULL;
	return (__temp21 = g_strconcat (document_get_basename (self), folder, NULL), (folder == NULL ? NULL : (folder = (g_free (folder), NULL))), __temp21);
	(folder == NULL ? NULL : (folder = (g_free (folder), NULL)));
}


char* document_uri_to_filename (char* uri)
{
	char* __temp22 = NULL;
	char* f = (__temp22 = uri, (__temp22 == NULL ? __temp22 : g_strdup (__temp22)));
	/* only local URIs are supported for now*/
	if (g_str_has_prefix (uri, "file:")) {
		char* __temp24 = NULL;
		char* __temp23 = NULL;
		f = (__temp24 = (__temp23 = g_strsplit (uri, "://", 0)[1], (__temp23 == NULL ? __temp23 : g_strdup (__temp23))), (f == NULL ? NULL : (f = (g_free (f), NULL))), __temp24);
	} else {
		if (!g_path_is_absolute (uri)) {
			char* __temp26 = NULL;
			char* __temp25 = NULL;
			f = (__temp26 = g_build_filename ((__temp25 = g_get_current_dir ()), uri, NULL), (f == NULL ? NULL : (f = (g_free (f), NULL))), __temp26);
			(__temp25 == NULL ? NULL : (__temp25 = (g_free (__temp25), NULL)));
		}
	}
	return f;
	(f == NULL ? NULL : (f = (g_free (f), NULL)));
}


MetadataManager* document_get_metadata (Document* self)
{
	g_return_val_if_fail (IS_DOCUMENT (self), NULL);
	return self->priv->_metadata;
}


void document_set_metadata (Document* self, MetadataManager* value)
{
	g_return_if_fail (IS_DOCUMENT (self));
	MetadataManager* __temp30 = NULL;
	MetadataManager* __temp29 = NULL;
	self->priv->_metadata = (__temp30 = (__temp29 = value, (__temp29 == NULL ? __temp29 : g_object_ref (__temp29))), (self->priv->_metadata == NULL ? NULL : (self->priv->_metadata = (g_object_unref (self->priv->_metadata), NULL))), __temp30);
}


static GObject * document_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties)
{
	GObject * obj;
	DocumentClass * klass;
	GObjectClass * parent_class;
	klass = DOCUMENT_CLASS (g_type_class_peek (TYPE_DOCUMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	Document * self = DOCUMENT (obj);
	{
		gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (self), TRUE);
	}
	return obj;
}


static void document_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	Document * self = DOCUMENT (object);
	switch (property_id) {
		case DOCUMENT_METADATA:
		g_value_set_object (value, document_get_metadata (self));
		break;
	}
}


static void document_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	Document * self = DOCUMENT (object);
	switch (property_id) {
		case DOCUMENT_METADATA:
		document_set_metadata (self, g_value_get_object (value));
		break;
	}
}


static void document_class_init (DocumentClass * klass)
{
	g_type_class_add_private (klass, sizeof (DocumentPrivate));
	G_OBJECT_CLASS (klass)->get_property = document_get_property;
	G_OBJECT_CLASS (klass)->set_property = document_set_property;
	G_OBJECT_CLASS (klass)->constructor = document_constructor;
	G_OBJECT_CLASS (klass)->dispose = document_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DOCUMENT_METADATA, g_param_spec_object ("metadata", "foo", "bar", TYPE_METADATA_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void document_init (Document * self)
{
	self->priv = DOCUMENT_GET_PRIVATE (self);
}


static void document_dispose (GObject * obj)
{
	Document * self = DOCUMENT (obj);
	(self->priv->file_name == NULL ? NULL : (self->priv->file_name = (g_free (self->priv->file_name), NULL)));
	(self->priv->mime_type == NULL ? NULL : (self->priv->mime_type = (g_free (self->priv->mime_type), NULL)));
	(self->priv->modified == NULL ? NULL : (self->priv->modified = (g_free (self->priv->modified), NULL)));
	(self->priv->_metadata == NULL ? NULL : (self->priv->_metadata = (g_object_unref (self->priv->_metadata), NULL)));
	DocumentClass * klass;
	GObjectClass * parent_class;
	klass = DOCUMENT_CLASS (g_type_class_peek (TYPE_DOCUMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType document_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (DocumentClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) document_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Document), 0, (GInstanceInitFunc) document_init };
		g_define_type_id = g_type_register_static (GTK_TYPE_SOURCE_BUFFER, "Document", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



