
#include "documentview.h"
#include <gtksourceview/gtksourceiter.h>
#include <stdio.h>
#include <pango/pango.h>
#include "metadata.h"
#include "basewindow.h"
#include "settings.h"

struct _TagPrivate {
};
#define TAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_TAG, TagPrivate))
enum  {
	TAG_DUMMY_PROPERTY,
};
static void tag_dispose (GObject * obj);
struct _DocumentViewPrivate {
	Document* _document;
	GSList* tag_list;
	gboolean modified;
};
#define DOCUMENT_VIEW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_DOCUMENT_VIEW, DocumentViewPrivate))
enum  {
	DOCUMENT_VIEW_DUMMY_PROPERTY,
	DOCUMENT_VIEW_DOCUMENT,
};
static void document_view_save_state (DocumentView* self);
static void document_view_restore_state (DocumentView* self);
static char* document_view_normalize_block_clip (DocumentView* self, char* clipstring);
static void document_view_save_selection (DocumentView* self);
static void document_view_restore_selection (DocumentView* self);
static void document_view_select_block (DocumentView* self, GtkTextIter* start, GtkTextIter* end);
static void document_view_add_tag (DocumentView* self, GtkTextIter s, GtkTextIter e);
static void document_view_update_tags (DocumentView* self);
static void document_view_on_font_changed (DocumentView* self, unsigned int cnxn_id, GConfEntry entry);
static gboolean document_view_on_focus_out (GObject* o, GdkEventFocus* e, DocumentView* self);
static void document_view_on_cut_clipboard (GObject* ow, DocumentView* self);
static void document_view_on_copy_clipboard (GObject* o, DocumentView* self);
static void document_view_on_paste_clipboard (GObject* o, DocumentView* self);
static GObject * document_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void document_view_dispose (GObject * obj);

static void tag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	Tag * self = TAG (object);
	switch (property_id) {
	}
}


static void tag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	Tag * self = TAG (object);
	switch (property_id) {
	}
}


static void tag_class_init (TagClass * klass)
{
	G_OBJECT_CLASS (klass)->get_property = tag_get_property;
	G_OBJECT_CLASS (klass)->set_property = tag_set_property;
	G_OBJECT_CLASS (klass)->dispose = tag_dispose;
}


static void tag_init (Tag * self)
{
}


static void tag_dispose (GObject * obj)
{
	Tag * self = TAG (obj);
	(self->begin == NULL ? NULL : (self->begin = (g_object_unref (self->begin), NULL)));
	(self->end == NULL ? NULL : (self->end = (g_object_unref (self->end), NULL)));
	TagClass * klass;
	GObjectClass * parent_class;
	klass = TAG_CLASS (g_type_class_peek (TYPE_TAG));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType tag_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (TagClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tag_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Tag), 0, (GInstanceInitFunc) tag_init };
		g_define_type_id = g_type_register_static (G_TYPE_OBJECT, "Tag", &g_define_type_info, 0);
	}
	return g_define_type_id;
}


DocumentView* document_view_new (Document* doc)
{
	g_return_val_if_fail (doc == NULL || IS_DOCUMENT (doc), NULL);
	GParameter * __params = g_new0 (GParameter, 0);
	GParameter * __params_it = __params;
	DocumentView * self = g_object_newv (TYPE_DOCUMENT_VIEW, __params_it - __params, __params);
	if (doc != NULL) {
		document_view_set_document (self, doc);
	}
	return self;
}


static void document_view_save_state (DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	if (gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &start_iter, &end_iter)) {
		int tmp = gtk_text_iter_get_offset (&end_iter);
		char* __temp0 = NULL;
		metadata_manager_set (document_get_metadata (document_view_get_document (self)), "sel_offset", (__temp0 = g_strdup_printf ("%i", tmp)));
		(__temp0 == NULL ? NULL : (__temp0 = (g_free (__temp0), NULL)));
	} else {
		metadata_manager_set (document_get_metadata (document_view_get_document (self)), "sel_offset", "-1");
	}
	int tmp = gtk_text_iter_get_offset (&start_iter);
	char* __temp1 = NULL;
	metadata_manager_set (document_get_metadata (document_view_get_document (self)), "cur_offset", (__temp1 = g_strdup_printf ("%i", tmp)));
	(__temp1 == NULL ? NULL : (__temp1 = (g_free (__temp1), NULL)));
}


static void document_view_restore_state (DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	int cur = metadata_manager_get_int (document_get_metadata (document_view_get_document (self)), "cur_offset");
	int sel = metadata_manager_get_int (document_get_metadata (document_view_get_document (self)), "sel_offset");
	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (document_view_get_document (self)), &start_iter, cur);
	if (sel > -1) {
		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (document_view_get_document (self)), &end_iter, sel);
	} else {
		end_iter = start_iter;
	}
	gtk_text_buffer_select_range (GTK_TEXT_BUFFER (document_view_get_document (self)), &start_iter, &end_iter);
}


gboolean document_view_find (DocumentView* self, char* text, GtkTextIter start, gboolean match_case, gboolean backwards, gboolean wrap)
{
	g_return_val_if_fail (IS_DOCUMENT_VIEW (self), FALSE);
	g_return_if_fail (document_view_get_document (self) != NULL);
	GtkTextIter match;
	GtkTextIter match_end;
	GtkSourceSearchFlags search_flags;
	gboolean wrapped = FALSE;
	gboolean found = FALSE;
	if (!match_case) {
		search_flags |= GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
	}
	GtkTextIter limit;
	if (backwards) {
		gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (document_view_get_document (self)), &limit);
	} else {
		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (document_view_get_document (self)), &limit);
	}
	while (TRUE) {
		if (backwards) {
			found = gtk_source_iter_backward_search (&start, text, search_flags, &match, &match_end, &limit);
		} else {
			found = gtk_source_iter_forward_search (&start, text, search_flags, &match, &match_end, &limit);
		}
		if (found) {
			gtk_text_buffer_select_range (GTK_TEXT_BUFFER (document_view_get_document (self)), &match, &match_end);
			gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (self), &match_end, 0, FALSE, 0, 0);
			return TRUE;
		} else {
			if (!wrap || wrapped) {
				return FALSE;
			}
		}
		if (wrap && !wrapped) {
			limit = start;
			if (backwards) {
				gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (document_view_get_document (self)), &start);
			} else {
				gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (document_view_get_document (self)), &start);
			}
			wrapped = TRUE;
		}
	}
	;
	return FALSE;
}


void document_view_select_all (DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	g_return_if_fail (document_view_get_document (self) != NULL);
	GtkTextIter s;
	GtkTextIter e;
	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, &e);
	gtk_text_buffer_select_range (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, &e);
}


static char* document_view_normalize_block_clip (DocumentView* self, char* clipstring)
{
	g_return_val_if_fail (IS_DOCUMENT_VIEW (self), NULL);
	char* __temp5 = NULL;
	char* ret = (__temp5 = clipstring, (__temp5 == NULL ? __temp5 : g_strdup (__temp5)));
	if (!g_str_has_prefix (clipstring, "\n")) {
		char* __temp6 = NULL;
		ret = (__temp6 = g_strconcat ("\n", ret, NULL), (ret == NULL ? NULL : (ret = (g_free (ret), NULL))), __temp6);
	}
	char* __temp7 = NULL;
	return (__temp7 = g_strconcat (ret, "\n", NULL), (ret == NULL ? NULL : (ret = (g_free (ret), NULL))), __temp7);
	(ret == NULL ? NULL : (ret = (g_free (ret), NULL)));
}


static void document_view_save_selection (DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter start;
	GtkTextIter end;
	gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &start, &end);
	gtk_text_buffer_create_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), "saved_insert", &start, TRUE);
	gtk_text_buffer_create_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), "saved_selection_bound", &end, TRUE);
}


static void document_view_restore_selection (DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter start;
	GtkTextIter end;
	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &start, gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), "saved_insert"));
	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &end, gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), "saved_selection_bound"));
	gtk_text_buffer_select_range (GTK_TEXT_BUFFER (document_view_get_document (self)), &start, &end);
}


static void document_view_select_block (DocumentView* self, GtkTextIter* start, GtkTextIter* end)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	gtk_text_iter_set_line_index (&*start, 0);
	if (!gtk_text_iter_ends_line (&*end)) {
		gtk_text_iter_forward_to_line_end (&*end);
	}
	if (gtk_text_iter_get_line (&*start) > 0) {
		gtk_text_iter_backward_char (&*start);
	} else {
		gtk_text_iter_forward_char (&*end);
	}
	gtk_text_buffer_select_range (GTK_TEXT_BUFFER (document_view_get_document (self)), &*start, &*end);
}


void document_view_block_cut_clipboard (DocumentView* self, gboolean block)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	g_return_if_fail (document_view_get_document (self) != NULL);
	GtkTextIter start;
	GtkTextIter end;
	gboolean has_selection = gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &start, &end);
	if (has_selection && !block) {
		gtk_text_buffer_cut_clipboard (GTK_TEXT_BUFFER (document_view_get_document (self)), base_window_get_clipboard (), TRUE);
	} else {
		int line = gtk_text_iter_get_line (&start);
		int offset = gtk_text_iter_get_line_offset (&start);
		document_view_select_block (self, &start, &end);
		char* clipstring = document_view_normalize_block_clip (self, gtk_text_buffer_get_text (GTK_TEXT_BUFFER (document_view_get_document (self)), &start, &end, FALSE));
		gtk_text_buffer_delete_interactive (GTK_TEXT_BUFFER (document_view_get_document (self)), &start, &end, TRUE);
		gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (document_view_get_document (self)), &start, line);
		if (offset > gtk_text_iter_get_chars_in_line (&start) - 1) {
			gtk_text_iter_set_line_offset (&start, gtk_text_iter_get_chars_in_line (&start) - 1);
		} else {
			gtk_text_iter_set_line_offset (&start, offset);
		}
		gtk_text_buffer_select_range (GTK_TEXT_BUFFER (document_view_get_document (self)), &start, &start);
		gtk_clipboard_set_text (base_window_get_clipboard (), clipstring, -1);
		(clipstring == NULL ? NULL : (clipstring = (g_free (clipstring), NULL)));
	}
}


void document_view_block_copy_clipboard (DocumentView* self, gboolean block)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	g_return_if_fail (document_view_get_document (self) != NULL);
	GtkTextIter start;
	GtkTextIter end;
	gboolean has_selection = gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &start, &end);
	if (has_selection && !block) {
		gtk_text_buffer_copy_clipboard (gtk_text_view_get_buffer (GTK_TEXT_VIEW (self)), base_window_get_clipboard ());
	} else {
		document_view_save_selection (self);
		document_view_select_block (self, &start, &end);
		document_view_restore_selection (self);
		char* clipstring = document_view_normalize_block_clip (self, gtk_text_buffer_get_text (GTK_TEXT_BUFFER (document_view_get_document (self)), &start, &end, FALSE));
		gtk_clipboard_set_text (base_window_get_clipboard (), clipstring, -1);
		(clipstring == NULL ? NULL : (clipstring = (g_free (clipstring), NULL)));
	}
}


void document_view_block_paste_clipboard (DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	g_return_if_fail (document_view_get_document (self) != NULL);
	GtkTextIter iter;
	GtkTextIter end;
	char* clipstring = gtk_clipboard_wait_for_text (base_window_get_clipboard ());
	gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &iter, &end);
	if (g_str_has_prefix (clipstring, "\n") && g_str_has_suffix (clipstring, "\n")) {
		GString* str = g_string_new (gtk_clipboard_wait_for_text (base_window_get_clipboard ()));
		g_string_erase (str, str->len - 1, 1);
		if (!gtk_text_iter_ends_line (&iter)) {
			gtk_text_iter_forward_to_line_end (&iter);
		}
		gtk_text_buffer_insert_interactive (GTK_TEXT_BUFFER (document_view_get_document (self)), &iter, str->str, -1, TRUE);
		(str == NULL ? NULL : (str = (g_string_free (str, TRUE), NULL)));
	} else {
		gtk_text_buffer_paste_clipboard (GTK_TEXT_BUFFER (document_view_get_document (self)), base_window_get_clipboard (), &iter, TRUE);
	}
}


int document_view_get_current_line (DocumentView* self)
{
	g_return_val_if_fail (IS_DOCUMENT_VIEW (self), 0);
	g_return_if_fail (document_view_get_document (self) != NULL);
	GtkTextIter iter;
	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &iter, gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (document_view_get_document (self))));
	return gtk_text_iter_get_line (&iter);
}


void document_view_go_to_line (DocumentView* self, int linenum)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter iter;
	if (linenum <= 0) {
		return;
	}
	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &iter, gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (document_view_get_document (self))));
	gtk_text_iter_set_line (&iter, linenum - 1);
	gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (document_view_get_document (self)), &iter);
	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (self), &iter, 0, FALSE, 0, 0);
}


void document_view_insert_new_line (DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter iter;
	gunichar ichar;
	GString* indent = g_string_new ("");
	char* tmp = NULL;
	/* save the indentation of the current line*/
	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &iter, gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (document_view_get_document (self))));
	gtk_text_iter_set_line_offset (&iter, 0);
	while (gtk_text_iter_get_char (&iter) == '\t' || gtk_text_iter_get_char (&iter) == ' ') {
		ichar = gtk_text_iter_get_char (&iter);
		g_string_append_unichar (indent, ichar);
		gtk_text_iter_forward_char (&iter);
	}
	/* move the cursor to the end of the line and insert a newline character with indentation*/
	if (!gtk_text_iter_ends_line (&iter)) {
		gtk_text_iter_forward_to_line_end (&iter);
	}
	char* __temp9 = NULL;
	gtk_text_buffer_insert_interactive (GTK_TEXT_BUFFER (document_view_get_document (self)), &iter, (__temp9 = g_strconcat ("\n", indent->str, NULL)), -1, TRUE);
	(__temp9 == NULL ? NULL : (__temp9 = (g_free (__temp9), NULL)));
	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &iter, gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (document_view_get_document (self))));
	if (!gtk_text_iter_ends_line (&iter)) {
		gtk_text_iter_forward_to_line_end (&iter);
		gtk_text_iter_forward_to_line_end (&iter);
	}
	gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (document_view_get_document (self)), &iter);
	(indent == NULL ? NULL : (indent = (g_string_free (indent, TRUE), NULL)));
}


void document_view_tag_selection (DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter s;
	GtkTextIter e;
	if (gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, &e)) {
		document_view_add_tag (self, s, e);
	}
}


void document_view_tag_and_find_next (DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter s;
	GtkTextIter e;
	if (gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, &e)) {
		document_view_add_tag (self, s, e);
		document_view_find (self, gtk_text_buffer_get_text (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, &e, FALSE), e, TRUE, FALSE, TRUE);
	}
}


void document_view_tag_search_results (DocumentView* self, char* str)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter s;
	GtkTextIter e;
	GtkTextIter match;
	GtkTextIter match_end;
	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, &e);
	while (gtk_source_iter_forward_search (&s, str, (GtkSourceSearchFlags) 0, &match, &match_end, &e)) {
		document_view_add_tag (self, match, match_end);
		s = match_end;
	}
}


void document_view_remove_tags (DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter s;
	GtkTextIter e;
	int os;
	int oe;
	int ts;
	int te;
	if (!gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, &e)) {
		gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, &e);
	}
	os = gtk_text_iter_get_offset (&s);
	oe = gtk_text_iter_get_offset (&e);
	GSList* tmp_list = NULL;
	{
		GSList* tag_it;
		for (tag_it = self->priv->tag_list; tag_it != NULL; tag_it = tag_it->next) {
			Tag* tag = tag_it->data;
			{
				gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, tag->begin);
				gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &e, tag->end);
				ts = gtk_text_iter_get_offset (&s);
				te = gtk_text_iter_get_offset (&e);
				if (ts < os || te > oe) {
					Tag* __temp10 = NULL;
					tmp_list = g_slist_append (tmp_list, (__temp10 = tag, (__temp10 == NULL ? __temp10 : g_object_ref (__temp10))));
				}
			}
		}
	}
	GSList* __temp12 = NULL;
	GSList* __temp11 = NULL;
	self->priv->tag_list = (__temp12 = (__temp11 = tmp_list, (__temp11 == NULL ? __temp11 : g_slist_copy (__temp11))), (self->priv->tag_list == NULL ? NULL : (self->priv->tag_list = (g_slist_free (self->priv->tag_list), NULL))), __temp12);
	document_view_update_tags (self);
}


void document_view_replace (DocumentView* self, char* str)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter s;
	GtkTextIter e;
	gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER (document_view_get_document (self)));
	{
		GSList* tag_it;
		for (tag_it = self->priv->tag_list; tag_it != NULL; tag_it = tag_it->next) {
			Tag* tag = tag_it->data;
			{
				gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, tag->begin);
				gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &e, tag->end);
				gtk_text_buffer_delete (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, &e);
				gtk_text_buffer_insert_interactive (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, str, -1, TRUE);
				gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, tag->begin);
				gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &e, tag->end);
				gtk_text_buffer_apply_tag_by_name (GTK_TEXT_BUFFER (document_view_get_document (self)), "tag", &s, &e);
			}
		}
	}
	gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (document_view_get_document (self)));
}


static void document_view_add_tag (DocumentView* self, GtkTextIter s, GtkTextIter e)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	gtk_text_buffer_apply_tag_by_name (GTK_TEXT_BUFFER (document_view_get_document (self)), "tag", &s, &e);
	Tag* tag = g_object_new (TYPE_TAG, NULL);
	GtkTextMark* __temp14 = NULL;
	GtkTextMark* __temp13 = NULL;
	tag->begin = (__temp14 = (__temp13 = gtk_text_buffer_create_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), NULL, &s, TRUE), (__temp13 == NULL ? __temp13 : g_object_ref (__temp13))), (tag->begin == NULL ? NULL : (tag->begin = (g_object_unref (tag->begin), NULL))), __temp14);
	GtkTextMark* __temp16 = NULL;
	GtkTextMark* __temp15 = NULL;
	tag->end = (__temp16 = (__temp15 = gtk_text_buffer_create_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), NULL, &e, FALSE), (__temp15 == NULL ? __temp15 : g_object_ref (__temp15))), (tag->end == NULL ? NULL : (tag->end = (g_object_unref (tag->end), NULL))), __temp16);
	Tag* __temp17 = NULL;
	self->priv->tag_list = g_slist_append (self->priv->tag_list, (__temp17 = tag, (__temp17 == NULL ? __temp17 : g_object_ref (__temp17))));
	(tag == NULL ? NULL : (tag = (g_object_unref (tag), NULL)));
}


static void document_view_update_tags (DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	GtkTextIter s;
	GtkTextIter e;
	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, &e);
	gtk_text_buffer_remove_tag_by_name (GTK_TEXT_BUFFER (document_view_get_document (self)), "tag", &s, &e);
	{
		GSList* tag_it;
		for (tag_it = self->priv->tag_list; tag_it != NULL; tag_it = tag_it->next) {
			Tag* tag = tag_it->data;
			{
				gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &s, tag->begin);
				gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self)), &e, tag->end);
				gtk_text_buffer_apply_tag_by_name (GTK_TEXT_BUFFER (document_view_get_document (self)), "tag", &s, &e);
			}
		}
	}
}


static void document_view_on_font_changed (DocumentView* self, unsigned int cnxn_id, GConfEntry entry)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	fprintf (stdout, "font changed\n", NULL);
}


static gboolean document_view_on_focus_out (GObject* o, GdkEventFocus* e, DocumentView* self)
{
	g_return_val_if_fail (IS_DOCUMENT_VIEW (self), FALSE);
	g_return_val_if_fail (o == NULL || G_IS_OBJECT (o), FALSE);
	g_return_if_fail (document_view_get_document (self) != NULL);
	/* save document state*/
	document_view_save_state (self);
	if (gtk_text_view_get_wrap_mode (GTK_TEXT_VIEW (self)) == GTK_WRAP_NONE) {
		metadata_manager_set (document_get_metadata (document_view_get_document (self)), "word-wrap", "false");
	} else {
		metadata_manager_set (document_get_metadata (document_view_get_document (self)), "word-wrap", "true");
	}
	metadata_manager_write (document_get_metadata (document_view_get_document (self)));
	document_save (document_view_get_document (self));
	return FALSE;
}


/* these need to be default handler overrides instead, otherwise things are getting pasted twice... */
static void document_view_on_cut_clipboard (GObject* ow, DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	g_return_if_fail (ow == NULL || G_IS_OBJECT (ow));
	fprintf (stdout, "cut\n", NULL);
}


/*block_cut_clipboard ();*/
static void document_view_on_copy_clipboard (GObject* o, DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	g_return_if_fail (o == NULL || G_IS_OBJECT (o));
	fprintf (stdout, "copy\n", NULL);
}


/*block_copy_clipboard ();*/
static void document_view_on_paste_clipboard (GObject* o, DocumentView* self)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	g_return_if_fail (o == NULL || G_IS_OBJECT (o));
	fprintf (stdout, "past\n", NULL);
}


Document* document_view_get_document (DocumentView* self)
{
	g_return_val_if_fail (IS_DOCUMENT_VIEW (self), NULL);
	return self->priv->_document;
}


void document_view_set_document (DocumentView* self, Document* value)
{
	g_return_if_fail (IS_DOCUMENT_VIEW (self));
	Document* __temp21 = NULL;
	Document* __temp20 = NULL;
	self->priv->_document = (__temp21 = (__temp20 = value, (__temp20 == NULL ? __temp20 : g_object_ref (__temp20))), (self->priv->_document == NULL ? NULL : (self->priv->_document = (g_object_unref (self->priv->_document), NULL))), __temp21);
	gtk_text_view_set_buffer (GTK_TEXT_VIEW (self), GTK_TEXT_BUFFER (self->priv->_document));
	GtkTextTag* tag = gtk_text_tag_new ("tag");
	char* __temp22 = NULL;
	g_object_set (tag, "background", "yellow", NULL);
	gtk_text_tag_table_add (gtk_text_buffer_get_tag_table (GTK_TEXT_BUFFER (document_view_get_document (self))), tag);
	document_view_restore_state (self);
	(tag == NULL ? NULL : (tag = (g_object_unref (tag), NULL)));
}


static GObject * document_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties)
{
	GObject * obj;
	DocumentViewClass * klass;
	GObjectClass * parent_class;
	klass = DOCUMENT_VIEW_CLASS (g_type_class_peek (TYPE_DOCUMENT_VIEW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	DocumentView * self = DOCUMENT_VIEW (obj);
	{
		gtk_widget_modify_font (GTK_WIDGET (self), pango_font_description_from_string (system_settings_get_font ()));
		/*GConf.Client.get_default().notify_add ("/desktop/gnome/interface/monospace_font_name", on_font_changed);*/
		gtk_source_view_set_tabs_width (GTK_SOURCE_VIEW (self), 4);
		gtk_source_view_set_indent_on_tab (GTK_SOURCE_VIEW (self), TRUE);
		gtk_source_view_set_highlight_current_line (GTK_SOURCE_VIEW (self), TRUE);
		gtk_source_view_set_auto_indent (GTK_SOURCE_VIEW (self), TRUE);
		gtk_source_view_set_show_line_numbers (GTK_SOURCE_VIEW (self), settings_get_show_line_numbers ());
		gtk_source_view_set_show_margin (GTK_SOURCE_VIEW (self), settings_get_show_margin ());
		g_signal_connect_object (self, "cut-clipboard", (GCallback) document_view_on_cut_clipboard, self, 0);
		g_signal_connect_object (self, "copy-clipboard", (GCallback) document_view_on_copy_clipboard, self, 0);
		g_signal_connect_object (self, "paste-clipboard", (GCallback) document_view_on_paste_clipboard, self, 0);
		g_signal_connect_object (self, "focus-out-event", (GCallback) document_view_on_focus_out, self, 0);
	}
	return obj;
}


static void document_view_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	DocumentView * self = DOCUMENT_VIEW (object);
	switch (property_id) {
		case DOCUMENT_VIEW_DOCUMENT:
		g_value_set_object (value, document_view_get_document (self));
		break;
	}
}


static void document_view_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	DocumentView * self = DOCUMENT_VIEW (object);
	switch (property_id) {
		case DOCUMENT_VIEW_DOCUMENT:
		document_view_set_document (self, g_value_get_object (value));
		break;
	}
}


static void document_view_class_init (DocumentViewClass * klass)
{
	g_type_class_add_private (klass, sizeof (DocumentViewPrivate));
	G_OBJECT_CLASS (klass)->get_property = document_view_get_property;
	G_OBJECT_CLASS (klass)->set_property = document_view_set_property;
	G_OBJECT_CLASS (klass)->constructor = document_view_constructor;
	G_OBJECT_CLASS (klass)->dispose = document_view_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DOCUMENT_VIEW_DOCUMENT, g_param_spec_object ("document", "foo", "bar", TYPE_DOCUMENT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void document_view_init (DocumentView * self)
{
	self->priv = DOCUMENT_VIEW_GET_PRIVATE (self);
}


static void document_view_dispose (GObject * obj)
{
	DocumentView * self = DOCUMENT_VIEW (obj);
	(self->priv->_document == NULL ? NULL : (self->priv->_document = (g_object_unref (self->priv->_document), NULL)));
	(self->priv->tag_list == NULL ? NULL : (self->priv->tag_list = (g_slist_free (self->priv->tag_list), NULL)));
	DocumentViewClass * klass;
	GObjectClass * parent_class;
	klass = DOCUMENT_VIEW_CLASS (g_type_class_peek (TYPE_DOCUMENT_VIEW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType document_view_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (DocumentViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) document_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DocumentView), 0, (GInstanceInitFunc) document_view_init };
		g_define_type_id = g_type_register_static (GTK_TYPE_SOURCE_VIEW, "DocumentView", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



