
#ifndef __DOCUMENTVIEW_H__
#define __DOCUMENTVIEW_H__

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gtksourceview/gtksourceview.h>
#include <stdlib.h>
#include <string.h>
#include <gconf/gconf-client.h>
#include <gdk/gdk.h>
#include "document.h"

G_BEGIN_DECLS


#define TYPE_TAG (tag_get_type ())
#define TAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TAG, Tag))
#define TAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TAG, TagClass))
#define IS_TAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TAG))
#define IS_TAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TAG))
#define TAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TAG, TagClass))

typedef struct _Tag Tag;
typedef struct _TagClass TagClass;
typedef struct _TagPrivate TagPrivate;

#define TYPE_DOCUMENT_VIEW (document_view_get_type ())
#define DOCUMENT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DOCUMENT_VIEW, DocumentView))
#define DOCUMENT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DOCUMENT_VIEW, DocumentViewClass))
#define IS_DOCUMENT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DOCUMENT_VIEW))
#define IS_DOCUMENT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DOCUMENT_VIEW))
#define DOCUMENT_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DOCUMENT_VIEW, DocumentViewClass))

typedef struct _DocumentView DocumentView;
typedef struct _DocumentViewClass DocumentViewClass;
typedef struct _DocumentViewPrivate DocumentViewPrivate;

struct _Tag {
	GObject parent;
	TagPrivate * priv;
	GtkTextMark* begin;
	GtkTextMark* end;
};
struct _TagClass {
	GObjectClass parent;
};
struct _DocumentView {
	GtkSourceView parent;
	DocumentViewPrivate * priv;
};
struct _DocumentViewClass {
	GtkSourceViewClass parent;
};

GType tag_get_type ();
DocumentView* document_view_new (Document* doc);
gboolean document_view_find (DocumentView* self, char* text, GtkTextIter start, gboolean match_case, gboolean backwards, gboolean wrap);
void document_view_select_all (DocumentView* self);
void document_view_block_cut_clipboard (DocumentView* self, gboolean block);
void document_view_block_copy_clipboard (DocumentView* self, gboolean block);
void document_view_block_paste_clipboard (DocumentView* self);
int document_view_get_current_line (DocumentView* self);
void document_view_go_to_line (DocumentView* self, int linenum);
void document_view_insert_new_line (DocumentView* self);
void document_view_tag_selection (DocumentView* self);
void document_view_tag_and_find_next (DocumentView* self);
void document_view_tag_search_results (DocumentView* self, char* str);
void document_view_remove_tags (DocumentView* self);
void document_view_replace (DocumentView* self, char* str);
Document* document_view_get_document (DocumentView* self);
void document_view_set_document (DocumentView* self, Document* value);
GType document_view_get_type ();

G_END_DECLS

#endif
