
#include "documentwindow.h"
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "document.h"
#include "documentview.h"
#include "metadata.h"

struct _DocumentWindowPrivate {
};
#define DOCUMENT_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_DOCUMENT_WINDOW, DocumentWindowPrivate))
enum  {
	DOCUMENT_WINDOW_DUMMY_PROPERTY,
};
static gboolean document_window_real_is_document (BaseWindow* base, char* uri);
static void document_window_restore_spatial_state (DocumentWindow* self);
static void document_window_save_spatial_state (DocumentWindow* self);
static void document_window_populate_window (DocumentWindow* self);
static gboolean __lambda0 (DocumentWindow* self);
static void document_window_connect_events (DocumentWindow* self);
static void document_window_dispose (GObject * obj);

DocumentWindow* document_window_new (char* filename)
{
	GParameter * __params = g_new0 (GParameter, 0);
	GParameter * __params_it = __params;
	DocumentWindow * self = g_object_newv (TYPE_DOCUMENT_WINDOW, __params_it - __params, __params);
	document_window_populate_window (self);
	Document* doc = document_new (filename);
	document_open (doc);
	document_view_set_document (BASE_WINDOW (self)->document_view, doc);
	document_window_connect_events (self);
	document_window_restore_spatial_state (self);
	base_window_restore_scroll_position (BASE_WINDOW (self));
	base_window_update_menu_state (BASE_WINDOW (self));
	char* __temp0 = NULL;
	gtk_window_set_title (GTK_WINDOW (self), (__temp0 = document_get_title_string (doc)));
	(__temp0 == NULL ? NULL : (__temp0 = (g_free (__temp0), NULL)));
	/* FIXME: should use document mimtype icon*/
	GdkPixbuf* icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "scratchpad", 48, (GtkIconLookupFlags) 0, NULL);
	gtk_window_set_icon (GTK_WINDOW (self), icon);
	(doc == NULL ? NULL : (doc = (g_object_unref (doc), NULL)));
	return self;
}


static gboolean document_window_real_is_document (BaseWindow* base, char* uri)
{
	DocumentWindow * self = DOCUMENT_WINDOW (base);
	char* __temp1 = NULL;
	gboolean __temp2;
	return (__temp2 = (g_utf8_collate (document_get_file_name (document_view_get_document (BASE_WINDOW (self)->document_view)), (__temp1 = document_uri_to_filename (uri))) == 0), (__temp1 == NULL ? NULL : (__temp1 = (g_free (__temp1), NULL))), __temp2);
}


static void document_window_restore_spatial_state (DocumentWindow* self)
{
	g_return_if_fail (IS_DOCUMENT_WINDOW (self));
	MetadataManager* m = document_get_metadata (document_view_get_document (BASE_WINDOW (self)->document_view));
	int x = metadata_manager_get_int (m, "x");
	int y = metadata_manager_get_int (m, "y");
	int width = metadata_manager_get_int (m, "width");
	int height = metadata_manager_get_int (m, "height");
	if (x != -1 && y != -1) {
		gtk_window_move (GTK_WINDOW (self), x, y);
	}
	if (width != -1 && height != -1) {
		gtk_window_resize (GTK_WINDOW (self), width, height);
	} else {
		gtk_window_resize (GTK_WINDOW (self), 780, 600);
	}
}


static void document_window_save_spatial_state (DocumentWindow* self)
{
	g_return_if_fail (IS_DOCUMENT_WINDOW (self));
	MetadataManager* m = document_get_metadata (document_view_get_document (BASE_WINDOW (self)->document_view));
	int width;
	int height;
	int x;
	int y;
	gtk_window_get_size (GTK_WINDOW (self), &width, &height);
	gtk_window_get_position (GTK_WINDOW (self), &x, &y);
	char* __temp4 = NULL;
	metadata_manager_set (m, "x", (__temp4 = g_strdup_printf ("%i", x)));
	(__temp4 == NULL ? NULL : (__temp4 = (g_free (__temp4), NULL)));
	char* __temp5 = NULL;
	metadata_manager_set (m, "y", (__temp5 = g_strdup_printf ("%i", y)));
	(__temp5 == NULL ? NULL : (__temp5 = (g_free (__temp5), NULL)));
	char* __temp6 = NULL;
	metadata_manager_set (m, "width", (__temp6 = g_strdup_printf ("%i", width)));
	(__temp6 == NULL ? NULL : (__temp6 = (g_free (__temp6), NULL)));
	char* __temp7 = NULL;
	metadata_manager_set (m, "height", (__temp7 = g_strdup_printf ("%i", height)));
	(__temp7 == NULL ? NULL : (__temp7 = (g_free (__temp7), NULL)));
	metadata_manager_write (m);
}


static void document_window_populate_window (DocumentWindow* self)
{
	g_return_if_fail (IS_DOCUMENT_WINDOW (self));
	/* temporary hack until virtual functions can be chained */
	base_window_populate_base_window (BASE_WINDOW (self));
	DocumentView* __temp8 = NULL;
	BASE_WINDOW (self)->document_view = (__temp8 = g_object_ref_sink (document_view_new (NULL)), (BASE_WINDOW (self)->document_view == NULL ? NULL : (BASE_WINDOW (self)->document_view = (g_object_unref (BASE_WINDOW (self)->document_view), NULL))), __temp8);
	gtk_container_add (GTK_CONTAINER (BASE_WINDOW (self)->scrolled_window), GTK_WIDGET (BASE_WINDOW (self)->document_view));
}


static gboolean __lambda0 (DocumentWindow* self)
{
	document_window_save_spatial_state (self);
	return FALSE;
}


static void document_window_connect_events (DocumentWindow* self)
{
	g_return_if_fail (IS_DOCUMENT_WINDOW (self));
	g_signal_connect_object (self, "delete-event", (GCallback) __lambda0, self, 0);
	base_window_connect_base_events (BASE_WINDOW (self));
}


static void document_window_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	DocumentWindow * self = DOCUMENT_WINDOW (object);
	switch (property_id) {
	}
}


static void document_window_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	DocumentWindow * self = DOCUMENT_WINDOW (object);
	switch (property_id) {
	}
}


static void document_window_class_init (DocumentWindowClass * klass)
{
	G_OBJECT_CLASS (klass)->get_property = document_window_get_property;
	G_OBJECT_CLASS (klass)->set_property = document_window_set_property;
	BASE_WINDOW_CLASS (klass)->is_document = document_window_real_is_document;
}


static void document_window_init (DocumentWindow * self)
{
}


static void document_window_dispose (GObject * obj)
{
	DocumentWindow * self = DOCUMENT_WINDOW (obj);
	DocumentWindowClass * klass;
	GObjectClass * parent_class;
	klass = DOCUMENT_WINDOW_CLASS (g_type_class_peek (TYPE_DOCUMENT_WINDOW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType document_window_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (DocumentWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) document_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DocumentWindow), 0, (GInstanceInitFunc) document_window_init };
		g_define_type_id = g_type_register_static (TYPE_BASE_WINDOW, "DocumentWindow", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



