
#include "metadata.h"
#include <stdio.h>
#include "application.h"

struct _MetadataManagerPrivate {
	char* file_name;
	GHashTable* keys;
	gboolean modified;
};
#define METADATA_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_METADATA_MANAGER, MetadataManagerPrivate))
enum  {
	METADATA_MANAGER_DUMMY_PROPERTY,
};
static const char* METADATA_MANAGER_keynames[] = {"x", "y", "width", "height", "hscroll", "vscroll", "cur_offset", "sel_offset", "word-wrap", NULL};
static void metadata_manager_dispose (GObject * obj);

MetadataManager* metadata_manager_new (char* hash)
{
	GParameter * __params = g_new0 (GParameter, 0);
	GParameter * __params_it = __params;
	MetadataManager * self = g_object_newv (TYPE_METADATA_MANAGER, __params_it - __params, __params);
	char* contents = NULL;
	char* __temp1 = NULL;
	char* __temp0 = NULL;
	self->priv->file_name = (__temp1 = g_strconcat ((__temp0 = g_strconcat (application_get_config_dir (), "/metadata/", NULL)), hash, NULL), (self->priv->file_name == NULL ? NULL : (self->priv->file_name = (g_free (self->priv->file_name), NULL))), __temp1);
	(__temp0 == NULL ? NULL : (__temp0 = (g_free (__temp0), NULL)));
	if (g_file_test (self->priv->file_name, G_FILE_TEST_EXISTS)) {
		g_file_get_contents (self->priv->file_name, &contents, NULL, NULL);
		char** lines = g_strsplit (contents, "\n", 0);
		{
			char** line_it;
			for (line_it = lines; *line_it != NULL; line_it = line_it + 1) {
				char* line = *line_it;
				{
					char** keyval = g_strsplit (line, ":", 0);
					if (keyval[0] == NULL || keyval[1] == NULL) {
						continue;
					}
					char* __temp3 = NULL;
					char* __temp2 = NULL;
					g_hash_table_insert (self->priv->keys, (__temp2 = keyval[0], (__temp2 == NULL ? __temp2 : g_strdup (__temp2))), (__temp3 = keyval[1], (__temp3 == NULL ? __temp3 : g_strdup (__temp3))));
					(keyval == NULL ? NULL : (keyval = (g_strfreev (keyval), NULL)));
				}
			}
		}
		(lines == NULL ? NULL : (lines = (g_strfreev (lines), NULL)));
	}
	return self;
}


char* metadata_manager_get (MetadataManager* self, char* key)
{
	g_return_val_if_fail (IS_METADATA_MANAGER (self), NULL);
	char* val = g_hash_table_lookup (self->priv->keys, key);
	return ((val != NULL) ? val : "");
}


int metadata_manager_get_int (MetadataManager* self, char* key)
{
	g_return_val_if_fail (IS_METADATA_MANAGER (self), 0);
	char* r = g_hash_table_lookup (self->priv->keys, key);
	if (g_hash_table_lookup (self->priv->keys, key) != NULL) {
		return atoi (r);
	} else {
		return -1;
	}
}


long metadata_manager_get_long (MetadataManager* self, char* key)
{
	g_return_val_if_fail (IS_METADATA_MANAGER (self), 0);
	char* r = g_hash_table_lookup (self->priv->keys, key);
	if (g_hash_table_lookup (self->priv->keys, key) != NULL) {
		return (long) strtoll (r, NULL, 0);
	} else {
		return -1;
	}
}


void metadata_manager_set (MetadataManager* self, char* key, char* value)
{
	g_return_if_fail (IS_METADATA_MANAGER (self));
	char* __temp10 = NULL;
	char* __temp9 = NULL;
	g_hash_table_replace (self->priv->keys, (__temp9 = key, (__temp9 == NULL ? __temp9 : g_strdup (__temp9))), (__temp10 = value, (__temp10 == NULL ? __temp10 : g_strdup (__temp10))));
	self->priv->modified = TRUE;
}


void metadata_manager_write (MetadataManager* self)
{
	g_return_if_fail (IS_METADATA_MANAGER (self));
	if (!self->priv->modified) {
		return;
	}
	char* output = g_strdup ("");
	GError* err = NULL;
	{
		char** key_it;
		for (key_it = METADATA_MANAGER_keynames; *key_it != NULL; key_it = key_it + 1) {
			char* key = *key_it;
			{
				char* val = g_hash_table_lookup (self->priv->keys, key);
				if (val != NULL) {
					char* __temp14 = NULL;
					char* __temp13 = NULL;
					char* __temp12 = NULL;
					char* __temp11 = NULL;
					output = (__temp14 = g_strconcat ((__temp13 = g_strconcat ((__temp12 = g_strconcat ((__temp11 = g_strconcat (output, key, NULL)), ":", NULL)), val, NULL)), "\n", NULL), (output == NULL ? NULL : (output = (g_free (output), NULL))), __temp14);
					(__temp13 == NULL ? NULL : (__temp13 = (g_free (__temp13), NULL)));
					(__temp12 == NULL ? NULL : (__temp12 = (g_free (__temp12), NULL)));
					(__temp11 == NULL ? NULL : (__temp11 = (g_free (__temp11), NULL)));
				}
			}
		}
	}
	if (!g_file_set_contents (self->priv->file_name, output, -1, &err)) {
		char* __temp16 = NULL;
		char* __temp15 = NULL;
		fprintf (stdout, (__temp16 = g_strconcat ((__temp15 = g_strconcat ("Warning: Could not save metadata: ", err->message, NULL)), "\n", NULL)), NULL);
		(__temp16 == NULL ? NULL : (__temp16 = (g_free (__temp16), NULL)));
		(__temp15 == NULL ? NULL : (__temp15 = (g_free (__temp15), NULL)));
	}
	self->priv->modified = FALSE;
	(output == NULL ? NULL : (output = (g_free (output), NULL)));
}


static void metadata_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	MetadataManager * self = METADATA_MANAGER (object);
	switch (property_id) {
	}
}


static void metadata_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	MetadataManager * self = METADATA_MANAGER (object);
	switch (property_id) {
	}
}


static void metadata_manager_class_init (MetadataManagerClass * klass)
{
	g_type_class_add_private (klass, sizeof (MetadataManagerPrivate));
	G_OBJECT_CLASS (klass)->get_property = metadata_manager_get_property;
	G_OBJECT_CLASS (klass)->set_property = metadata_manager_set_property;
	G_OBJECT_CLASS (klass)->dispose = metadata_manager_dispose;
}


static void metadata_manager_init (MetadataManager * self)
{
	self->priv = METADATA_MANAGER_GET_PRIVATE (self);
	self->priv->keys = g_hash_table_new (g_str_hash, g_str_equal);
	self->priv->modified = FALSE;
}


static void metadata_manager_dispose (GObject * obj)
{
	MetadataManager * self = METADATA_MANAGER (obj);
	(self->priv->file_name == NULL ? NULL : (self->priv->file_name = (g_free (self->priv->file_name), NULL)));
	(self->priv->keys == NULL ? NULL : (self->priv->keys = (g_hash_table_unref (self->priv->keys), NULL)));
	MetadataManagerClass * klass;
	GObjectClass * parent_class;
	klass = METADATA_MANAGER_CLASS (g_type_class_peek (TYPE_METADATA_MANAGER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType metadata_manager_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (MetadataManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) metadata_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MetadataManager), 0, (GInstanceInitFunc) metadata_manager_init };
		g_define_type_id = g_type_register_static (G_TYPE_OBJECT, "MetadataManager", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



