
#include "searchentry.h"

struct _SearchEventArgsPrivate {
	SearchMode _mode;
};
#define SEARCH_EVENT_ARGS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SEARCH_EVENT_ARGS, SearchEventArgsPrivate))
enum  {
	SEARCH_EVENT_ARGS_DUMMY_PROPERTY,
	SEARCH_EVENT_ARGS_MODE,
};
static void search_event_args_dispose (GObject * obj);
struct _SearchEntryPrivate {
	SearchMode mode;
	GtkLabel* desc_label;
	int maxlines;
	char* last_search;
	char* last_replace;
};
#define SEARCH_ENTRY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SEARCH_ENTRY, SearchEntryPrivate))
enum  {
	SEARCH_ENTRY_DUMMY_PROPERTY,
};
static gboolean search_entry_on_changed (gpointer sender, SearchEntry* self);
static int search_entry_clamp (SearchEntry* self, int val);
static gboolean search_entry_on_key_press_event (gpointer sender, GdkEventKey* key, SearchEntry* self);
static void search_entry_dispose (GObject * obj);

SearchMode search_event_args_get_mode (SearchEventArgs* self)
{
	g_return_val_if_fail (IS_SEARCH_EVENT_ARGS (self), 0);
	return self->priv->_mode;
}


void search_event_args_set_mode (SearchEventArgs* self, SearchMode value)
{
	g_return_if_fail (IS_SEARCH_EVENT_ARGS (self));
	self->priv->_mode = value;
}


static void search_event_args_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	SearchEventArgs * self = SEARCH_EVENT_ARGS (object);
	switch (property_id) {
		case SEARCH_EVENT_ARGS_MODE:
		g_value_set_int (value, search_event_args_get_mode (self));
		break;
	}
}


static void search_event_args_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	SearchEventArgs * self = SEARCH_EVENT_ARGS (object);
	switch (property_id) {
		case SEARCH_EVENT_ARGS_MODE:
		search_event_args_set_mode (self, g_value_get_int (value));
		break;
	}
}


static void search_event_args_class_init (SearchEventArgsClass * klass)
{
	g_type_class_add_private (klass, sizeof (SearchEventArgsPrivate));
	G_OBJECT_CLASS (klass)->get_property = search_event_args_get_property;
	G_OBJECT_CLASS (klass)->set_property = search_event_args_set_property;
	G_OBJECT_CLASS (klass)->dispose = search_event_args_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), SEARCH_EVENT_ARGS_MODE, g_param_spec_int ("mode", "foo", "bar", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void search_event_args_init (SearchEventArgs * self)
{
	self->priv = SEARCH_EVENT_ARGS_GET_PRIVATE (self);
	self->backwards = FALSE;
	self->next_result = FALSE;
}


static void search_event_args_dispose (GObject * obj)
{
	SearchEventArgs * self = SEARCH_EVENT_ARGS (obj);
	(self->search_text == NULL ? NULL : (self->search_text = (g_free (self->search_text), NULL)));
	SearchEventArgsClass * klass;
	GObjectClass * parent_class;
	klass = SEARCH_EVENT_ARGS_CLASS (g_type_class_peek (TYPE_SEARCH_EVENT_ARGS));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType search_event_args_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (SearchEventArgsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) search_event_args_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SearchEventArgs), 0, (GInstanceInitFunc) search_event_args_init };
		g_define_type_id = g_type_register_static (G_TYPE_OBJECT, "SearchEventArgs", &g_define_type_info, 0);
	}
	return g_define_type_id;
}


SearchEntry* search_entry_new (GtkLabel* lbl)
{
	g_return_val_if_fail (lbl == NULL || GTK_IS_LABEL (lbl), NULL);
	GParameter * __params = g_new0 (GParameter, 0);
	GParameter * __params_it = __params;
	SearchEntry * self = g_object_newv (TYPE_SEARCH_ENTRY, __params_it - __params, __params);
	GtkLabel* __temp2 = NULL;
	GtkLabel* __temp1 = NULL;
	self->priv->desc_label = (__temp2 = (__temp1 = lbl, (__temp1 == NULL ? __temp1 : g_object_ref (__temp1))), (self->priv->desc_label == NULL ? NULL : (self->priv->desc_label = (g_object_unref (self->priv->desc_label), NULL))), __temp2);
	g_signal_connect_object (self, "key-press-event", (GCallback) search_entry_on_key_press_event, self, 0);
	g_signal_connect_object (self, "changed", (GCallback) search_entry_on_changed, self, 0);
	return self;
}


char* search_entry_get_last_search (SearchEntry* self)
{
	g_return_val_if_fail (IS_SEARCH_ENTRY (self), NULL);
	return self->priv->last_search;
}


char* search_entry_get_last_replace (SearchEntry* self)
{
	g_return_val_if_fail (IS_SEARCH_ENTRY (self), NULL);
	return self->priv->last_replace;
}


void search_entry_set_maxlines (SearchEntry* self, int lines)
{
	g_return_if_fail (IS_SEARCH_ENTRY (self));
	self->priv->maxlines = lines;
}


void search_entry_set_mode (SearchEntry* self, SearchMode new_mode)
{
	g_return_if_fail (IS_SEARCH_ENTRY (self));
	self->priv->mode = SEARCH_MODE_SILENT;
	if (new_mode == SEARCH_MODE_TEXT) {
		gtk_label_set_text (self->priv->desc_label, "Find:");
		gtk_entry_set_text (GTK_ENTRY (self), self->priv->last_search);
	} else {
		if (new_mode == SEARCH_MODE_GO_TO_LINE) {
			gtk_label_set_text (self->priv->desc_label, "Go to:");
		} else {
			if (new_mode == SEARCH_MODE_REPLACE) {
				gtk_label_set_text (self->priv->desc_label, "Replace:");
				gtk_entry_set_text (GTK_ENTRY (self), self->priv->last_replace);
			}
		}
	}
	self->priv->mode = new_mode;
}


static gboolean search_entry_on_changed (gpointer sender, SearchEntry* self)
{
	g_return_val_if_fail (IS_SEARCH_ENTRY (self), FALSE);
	if (self->priv->mode == SEARCH_MODE_SILENT) {
		return FALSE;
	}
	/* we don't replace-as-you-type as it would fill up the undo stack*/
	if (self->priv->mode == SEARCH_MODE_REPLACE) {
		return FALSE;
	}
	SearchEventArgs* args = g_object_new (TYPE_SEARCH_EVENT_ARGS, NULL);
	search_event_args_set_mode (args, self->priv->mode);
	char* __temp8 = NULL;
	char* __temp7 = NULL;
	args->search_text = (__temp8 = (__temp7 = gtk_entry_get_text (GTK_ENTRY (self)), (__temp7 == NULL ? __temp7 : g_strdup (__temp7))), (args->search_text == NULL ? NULL : (args->search_text = (g_free (args->search_text), NULL))), __temp8);
	if (self->priv->mode == SEARCH_MODE_TEXT) {
		char* __temp10 = NULL;
		char* __temp9 = NULL;
		self->priv->last_search = (__temp10 = (__temp9 = args->search_text, (__temp9 == NULL ? __temp9 : g_strdup (__temp9))), (self->priv->last_search == NULL ? NULL : (self->priv->last_search = (g_free (self->priv->last_search), NULL))), __temp10);
	} else {
		if (self->priv->mode == SEARCH_MODE_REPLACE) {
			char* __temp12 = NULL;
			char* __temp11 = NULL;
			self->priv->last_replace = (__temp12 = (__temp11 = args->search_text, (__temp11 == NULL ? __temp11 : g_strdup (__temp11))), (self->priv->last_replace == NULL ? NULL : (self->priv->last_replace = (g_free (self->priv->last_replace), NULL))), __temp12);
		}
	}
	g_signal_emit_by_name (G_OBJECT (self), "search-activated", args);
	gboolean __temp13;
	return (__temp13 = FALSE, (args == NULL ? NULL : (args = (g_object_unref (args), NULL))), __temp13);
	(args == NULL ? NULL : (args = (g_object_unref (args), NULL)));
}


static int search_entry_clamp (SearchEntry* self, int val)
{
	g_return_val_if_fail (IS_SEARCH_ENTRY (self), 0);
	if (val < 1) {
		return 1;
	}
	if (val > self->priv->maxlines) {
		return self->priv->maxlines;
	}
	return val;
}


static gboolean search_entry_on_key_press_event (gpointer sender, GdkEventKey* key, SearchEntry* self)
{
	g_return_val_if_fail (IS_SEARCH_ENTRY (self), FALSE);
	if (self->priv->mode == SEARCH_MODE_SILENT) {
		return FALSE;
	}
	SearchEventArgs* args = g_object_new (TYPE_SEARCH_EVENT_ARGS, NULL);
	search_event_args_set_mode (args, self->priv->mode);
	args->next_result = TRUE;
	char* __temp19 = NULL;
	char* __temp18 = NULL;
	args->search_text = (__temp19 = (__temp18 = gtk_entry_get_text (GTK_ENTRY (self)), (__temp18 == NULL ? __temp18 : g_strdup (__temp18))), (args->search_text == NULL ? NULL : (args->search_text = (g_free (args->search_text), NULL))), __temp19);
	if (self->priv->mode == SEARCH_MODE_TEXT) {
		char* __temp21 = NULL;
		char* __temp20 = NULL;
		self->priv->last_search = (__temp21 = (__temp20 = args->search_text, (__temp20 == NULL ? __temp20 : g_strdup (__temp20))), (self->priv->last_search == NULL ? NULL : (self->priv->last_search = (g_free (self->priv->last_search), NULL))), __temp21);
	} else {
		if (self->priv->mode == SEARCH_MODE_REPLACE) {
			char* __temp23 = NULL;
			char* __temp22 = NULL;
			self->priv->last_replace = (__temp23 = (__temp22 = args->search_text, (__temp22 == NULL ? __temp22 : g_strdup (__temp22))), (self->priv->last_replace == NULL ? NULL : (self->priv->last_replace = (g_free (self->priv->last_replace), NULL))), __temp23);
		}
	}
	if (g_utf8_collate (gdk_keyval_name (key->keyval), "Return") == 0 || g_utf8_collate (gdk_keyval_name (key->keyval), "Down") == 0 || g_utf8_collate (gdk_keyval_name (key->keyval), "Up") == 0) {
		/* we don't replace-as-you-type as it would fill up the undo stack*/
		if (self->priv->mode == SEARCH_MODE_REPLACE) {
			if (g_utf8_collate (gdk_keyval_name (key->keyval), "Return") == 0) {
				g_signal_emit_by_name (G_OBJECT (self), "search-activated", args);
				g_signal_emit_by_name (G_OBJECT (self), "search-closed");
				gboolean __temp24;
				return (__temp24 = TRUE, (args == NULL ? NULL : (args = (g_object_unref (args), NULL))), __temp24);
			}
			gboolean __temp25;
			return (__temp25 = FALSE, (args == NULL ? NULL : (args = (g_object_unref (args), NULL))), __temp25);
		}
		if (self->priv->mode == SEARCH_MODE_GO_TO_LINE) {
			if (g_utf8_collate (gdk_keyval_name (key->keyval), "Return") == 0) {
				g_signal_emit_by_name (G_OBJECT (self), "search-closed");
				gboolean __temp26;
				return (__temp26 = TRUE, (args == NULL ? NULL : (args = (g_object_unref (args), NULL))), __temp26);
			}
			int adjust = 0;
			if (g_utf8_collate (gdk_keyval_name (key->keyval), "Up") == 0) {
				adjust = 10;
			}
			if (g_utf8_collate (gdk_keyval_name (key->keyval), "Down") == 0) {
				adjust = -10;
			}
			if (key->state == 1) {
				/*shift pressed*/
				adjust *= 10;
			}
			/* great fun!*/
			char* __temp27 = NULL;
			gtk_entry_set_text (GTK_ENTRY (self), (__temp27 = g_strdup_printf ("%i", search_entry_clamp (self, (atoi (gtk_entry_get_text (GTK_ENTRY (self))) + adjust)))));
			(__temp27 == NULL ? NULL : (__temp27 = (g_free (__temp27), NULL)));
			gtk_widget_grab_focus (GTK_WIDGET (self));
			gboolean __temp28;
			return (__temp28 = TRUE, (args == NULL ? NULL : (args = (g_object_unref (args), NULL))), __temp28);
		}
		if (g_utf8_collate (gdk_keyval_name (key->keyval), "Up") == 0 || key->state == 1) {
			/*shift pressed*/
			args->backwards = TRUE;
		}
		g_signal_emit_by_name (G_OBJECT (self), "search-activated", args);
		gboolean __temp29;
		return (__temp29 = TRUE, (args == NULL ? NULL : (args = (g_object_unref (args), NULL))), __temp29);
	}
	gboolean __temp30;
	return (__temp30 = FALSE, (args == NULL ? NULL : (args = (g_object_unref (args), NULL))), __temp30);
	(args == NULL ? NULL : (args = (g_object_unref (args), NULL)));
}


static void search_entry_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	SearchEntry * self = SEARCH_ENTRY (object);
	switch (property_id) {
	}
}


static void search_entry_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	SearchEntry * self = SEARCH_ENTRY (object);
	switch (property_id) {
	}
}


static void search_entry_class_init (SearchEntryClass * klass)
{
	g_type_class_add_private (klass, sizeof (SearchEntryPrivate));
	G_OBJECT_CLASS (klass)->get_property = search_entry_get_property;
	G_OBJECT_CLASS (klass)->set_property = search_entry_set_property;
	G_OBJECT_CLASS (klass)->dispose = search_entry_dispose;
	g_signal_new ("search_activated", TYPE_SEARCH_ENTRY, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, TYPE_SEARCH_EVENT_ARGS);
	g_signal_new ("search_closed", TYPE_SEARCH_ENTRY, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void search_entry_init (SearchEntry * self)
{
	self->priv = SEARCH_ENTRY_GET_PRIVATE (self);
	self->priv->mode = SEARCH_MODE_TEXT;
	self->priv->maxlines = 9999;
	self->priv->last_search = g_strdup ("");
	self->priv->last_replace = g_strdup ("");
}


static void search_entry_dispose (GObject * obj)
{
	SearchEntry * self = SEARCH_ENTRY (obj);
	(self->priv->desc_label == NULL ? NULL : (self->priv->desc_label = (g_object_unref (self->priv->desc_label), NULL)));
	(self->priv->last_search == NULL ? NULL : (self->priv->last_search = (g_free (self->priv->last_search), NULL)));
	(self->priv->last_replace == NULL ? NULL : (self->priv->last_replace = (g_free (self->priv->last_replace), NULL)));
	SearchEntryClass * klass;
	GObjectClass * parent_class;
	klass = SEARCH_ENTRY_CLASS (g_type_class_peek (TYPE_SEARCH_ENTRY));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType search_entry_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (SearchEntryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) search_entry_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SearchEntry), 0, (GInstanceInitFunc) search_entry_init };
		g_define_type_id = g_type_register_static (GTK_TYPE_ENTRY, "SearchEntry", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



